from armorer.tools.workflow import workflow

def test_workflow_import():
    assert callable(workflow)
    print("Workflow tool imported successfully.")

if __name__ == "__main__":
    test_workflow_import()
